@@ -1,5 +1,6 @@
 import itertools
 import dedupe
+import predicates as predicates
 import numpy
 from collections import defaultdict
 
@@ -31,6 +32,10 @@ class Variable(object) :
 
         if definition.get('has missing', False) :
             self.has_missing = True
+            try :
+                self._predicate_functions += (predicates.existsPredicate,)
+            except AttributeError :
+                pass
         else :
             self.has_missing = False
 
@@ -45,13 +50,13 @@ class FieldType(Variable) :
         else :
             self.name = "(%s: %s)" % (self.field, self.type)
 
-        self.predicates = [dedupe.predicates.SimplePredicate(pred, self.field) 
+        self.predicates = [predicates.SimplePredicate(pred, self.field) 
                            for pred in self._predicate_functions]
 
         super(FieldType, self).__init__(definition)
 
 class ExactType(FieldType) :
-    _predicate_functions = [dedupe.predicates.wholeFieldPredicate]
+    _predicate_functions = [predicates.wholeFieldPredicate]
     type = "Exact"
 
     @staticmethod
@@ -81,7 +86,13 @@ class ShortStringType(FieldType) :
                             dedupe.predicates.sameFiveCharStartPredicate,
                             dedupe.predicates.sameSevenCharStartPredicate,
                             dedupe.predicates.commonFourGram,
-                            dedupe.predicates.commonSixGram)
+                            dedupe.predicates.commonSixGram,
+                            dedupe.predicates.commonTwoTokens,
+                            dedupe.predicates.commonThreeTokens,
+                            dedupe.predicates.fingerprint,
+                            dedupe.predicates.oneGramFingerprint,
+                            dedupe.predicates.twoGramFingerprint,
+                            dedupe.predicates.sortedAcronym)
 
 class StringType(ShortStringType) :
     comparator = normalizedAffineGapDistance
@@ -92,7 +103,7 @@ class StringType(ShortStringType) :
     def __init__(self, definition) :
         super(StringType, self).__init__(definition)
 
-        canopy_predicates = [dedupe.predicates.TfidfPredicate(threshold, 
+        canopy_predicates = [predicates.TfidfPredicate(threshold, 
                                                             self.field)
                              for threshold in self._canopy_thresholds]
 
@@ -113,21 +124,25 @@ class LatLongType(FieldType) :
     comparator = compareLatLong
     type = "LatLong"
 
-    _predicate_functions = [dedupe.predicates.latLongGridPredicate]
+    _predicate_functions = [predicates.latLongGridPredicate]
 
 class SetType(FieldType) :
     type = "Set"
 
     _predicate_functions = (dedupe.predicates.wholeSetPredicate,
-                         dedupe.predicates.commonSetElementPredicate)
-
+                            dedupe.predicates.commonSetElementPredicate,
+                            dedupe.predicates.lastSetElementPredicate,
+                            dedupe.predicates.commonTwoElementsPredicate,
+                            dedupe.predicates.commonThreeElementsPredicate,
+                            dedupe.predicates.firstSetElementPredicate)
+    
     _canopy_thresholds = (0.2, 0.4, 0.6, 0.8)
 
     def __init__(self, definition) :
         super(SetType, self).__init__(definition)
 
-        canopy_predicates = [dedupe.predicates.TfidfPredicate(threshold, 
-                                                               self.field)
+        canopy_predicates = [predicates.TfidfPredicate(threshold, 
+                                                       self.field)
                              for threshold in self._canopy_thresholds]
 
         self.predicates += canopy_predicates
@@ -140,7 +155,7 @@ class SetType(FieldType) :
 
 class CategoricalType(FieldType) :
     type = "Categorical"
-    _predicate_functions = [dedupe.predicates.wholeFieldPredicate]
+    _predicate_functions = [predicates.wholeFieldPredicate]
 
     def _categories(self, definition) :
         try :
@@ -167,6 +182,36 @@ class CategoricalType(FieldType) :
                                             'has missing' : self.has_missing})
             self.dummies.append(dummy_object)
 
+
+class ExistsType(FieldType) :
+    type = "Exists"
+    _predicate_functions = [predicates.existsPredicate]
+
+    def __init__(self, definition) :
+
+        super(ExistsType, self ).__init__(definition)
+        
+        self.cat_comparator = CategoricalComparator([0, 1])
+
+        self.dummies = []
+
+        for value, combo in sorted(self.cat_comparator.combinations[2:]) :
+            dummy_object = HigherDummyType({'combo' : combo, 
+                                            'value' : value,
+                                            'base name' : self.name,
+                                            'has missing' : self.has_missing})
+            self.dummies.append(dummy_object)
+
+    def comparator(self, field_1, field_2) :
+        if field_1 and field_2 :
+            return self.cat_comparator(1, 1)
+        elif field_1 or field_2 :
+            return self.cat_comparator(0, 1)
+        else :
+            return self.cat_comparator(0, 0)
+
+
+
 class SourceType(CategoricalType) :
     type = "Source"
 
